/*
 * Slawas.pl Copyright &copy; 2011-2012 
 * http://slawas.pl 
 * All rights reserved.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL SŁAWOMIR CICHY BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.scbpm.integrations;

import com.lombardisoftware.client.persistence.UserGroup;
import com.lombardisoftware.client.persistence.UserGroupFactory;
import com.lombardisoftware.core.TeamWorksException;
import com.lombardisoftware.server.core.GroupCore;

/**
 * ADGroup integrator operacji na grupach/rolach użytkowników.
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.1 $
 * 
 */
public class ADGroup {

	/**
	 * Komunikat jaki może zostać zwrócony podczas wykonywania operacji na
	 * grupach, oznaczający operację zakończoną sukcesem.
	 */
	public final static String SUCCESS = "SUCCESS";

	/**
	 * Komunikat jaki może zostać zwrócony podczas wykonywania operacji na
	 * grupach, oznaczający operację zakończoną błędem, gdy chcemy wykonać daną
	 * operację na grupie, która nie istnieje.
	 */
	public final static String GROUP_NOT_EXISTS = "GROUP_NOT_EXISTS";

	/**
	 * Komunikat jaki może zostać zwrócony podczas wykonywania operacji na
	 * grupach, oznaczający operację zakończoną błędem, gdy chcemy wykonać daną
	 * operację na grupie, która ma zdefiniowaną rolę manager'a, ale ta
	 * rola/grupa manager'a nie istnieje w systemie.
	 */
	public final static String PARENT_GROUP_NOT_EXISTS = "PARENT_GROUP_NOT_EXISTS";

	/**
	 * Metoda integracyjna zapisująca zmiany w grupie/tworząca nową grupę
	 * 
	 * @param groupName
	 *            nazwa grupy
	 * @param displayName
	 *            nazwa prezentacyjna grupy
	 * @param description
	 *            opis grupy
	 * @param groupType
	 *            typ grupy [{@link UserGroup#GROUP_TYPE_DYNAMIC}|
	 *            {@link UserGroup#GROUP_TYPE_SECURITY}|
	 *            {@link UserGroup#GROUP_TYPE_TEMPORARY}|
	 *            {@link UserGroup#GROUP_TYPE_INTERNAL}|
	 *            {@link UserGroup#GROUP_TYPE_PARTICIPANT}]
	 * @param parentGroupName
	 *            nazwa grupy manager'a
	 * @param skipCheck
	 *            flaga sterująca przyjmująca wartości {@code true} albo
	 *            {@code false}. Jeżeli ustawimy ją na {@code true} i grupa nie
	 *            istnieje w systemie to grupa ta oraz ewentualna grupa
	 *            manager'a zostanie utworzona, w przeciwnym wypadku operacja
	 *            będzie wykonywana tylko i wyłącznie na grupach już
	 *            istniejących.
	 * @return komunikat związany z sukcesem/porażką operacji. Przyjmuje
	 *         wartości [{@link #SUCCESS}|{@link #GROUP_NOT_EXISTS}|
	 *         {@link #PARENT_GROUP_NOT_EXISTS}]
	 * @throws TeamWorksException
	 */
	public static String saveGroup(String groupName, String displayName,
			String description, Integer groupType, String parentGroupName,
			boolean skipCheck) throws TeamWorksException {
		UserGroup group = GroupCore.getOrCreateGroup(groupName, skipCheck);
		if (group == null) {
			return GROUP_NOT_EXISTS;
		}
		group.setDescription(description);
		group.setDisplayName(displayName);
		group.setGroupType(groupType);
		if (parentGroupName != null && !"null".equals(parentGroupName)
				&& !"".equals(parentGroupName)) {
			UserGroup parentGroup = GroupCore.getOrCreateGroup(parentGroupName,
					skipCheck);
			if (parentGroup == null) {
				return PARENT_GROUP_NOT_EXISTS;
			}
			group.setParentGroupId(parentGroup.getGroupId());
		}
		UserGroupFactory.getInstance().save(group);
		return SUCCESS;
	}

}
