/*
 * Slawas.pl Copyright &copy; 2011-2012 
 * http://slawas.pl 
 * All rights reserved.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL SŁAWOMIR CICHY BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.scbpm.integrations;

import org.apache.commons.lang.StringUtils;
import org.scbpm.lib.api.UserSearcherFactory;
import org.scbpm.lib.ldap.exceptions.FactoryNewInstanceException;
import org.scbpm.lib.ldap.provider.ProviderOptions;

/**
 * UserSearcher
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.1 $
 * 
 */
public class UserSearcher {

	public UserSearcher() {
	}

	/**
	 * Wyszukiwanie w bazie danych IBM BPM na podstawie kryteriów zadanych w
	 * postaci filtra LDAP.
	 * 
	 * @see org.scbpm.lib.core.search.UserSearcher
	 * 
	 * @param ldapOptions
	 *            XML konfiguracyjny w postaci łańcucha znakowego.
	 * @param searchFilter
	 *            kryterium wyszukiwania w postaci filtra LDAP
	 * @param maxRecords
	 *            maksymalna liczba rekordów zwrócona w wyniku.
	 * @param returnType
	 *            typ do jakiego ma być skonwertowany wynik
	 * @return wynik odpowiedniego typu
	 * @throws Exception
	 */
	public static Object serachInBpm(String ldapOptions, String searchFilter,
			Integer maxRecords, String returnType) throws Exception,
			FactoryNewInstanceException {
		ProviderOptions po = ProviderOptions.getNewInstance(ldapOptions);
		return UserSearcherFactory.getFactory().getNewInstance(po)
				.serachInBpm(searchFilter, maxRecords, returnType);
	}

	/**
	 * Wyszukiwanie w bazie domyślnego repozytorium LDAP na podstawie kryteriów
	 * zadanych w postaci filtra LDAP.
	 * 
	 * @see org.scbpm.lib.core.search.UserSearcher
	 * 
	 * @param ldapOptions
	 *            XML konfiguracyjny w postaci łańcucha znakowego.
	 * @param organizationalUnitNames
	 *            lista nazw jednostek organizacyjnych odseparowanych znakiem
	 *            przecinka (może być pusta, czyli {@code null}), wtedy będzie
	 *            korzystać z domyślnych ctxDN zdefiniowanych w
	 *            {@link ProviderOptions#getBaseCtxDN()} oraz
	 *            {@link ProviderOptions#getRolesCtxDN()}.
	 * @param searchFilter
	 *            kryterium wyszukiwania w postaci filtra LDAP
	 * @param maxRecords
	 *            maksymalna liczba rekordów zwrócona w wyniku.
	 * @param returnType
	 *            typ do jakiego ma być skonwertowany wynik
	 * @return wynik odpowiedniego typu
	 * @throws Exception
	 */
	public static Object serachInLdap(String ldapOptions,
			String organizationalUnitNames, String searchFilter,
			Integer maxRecords, String returnType) throws Exception,
			FactoryNewInstanceException {
		ProviderOptions po = ProviderOptions.getNewInstance(ldapOptions);
		String organizationalUnitArray[] = null;
		if (StringUtils.isBlank(organizationalUnitNames)) {
			organizationalUnitArray = new String[1];
		} else {
			organizationalUnitArray = organizationalUnitNames.split("\\,");
			for (int i = 0; i < organizationalUnitArray.length; i++)
				organizationalUnitArray[i] = StringUtils
						.trim(organizationalUnitArray[i]);

		}
		return UserSearcherFactory
				.getFactory()
				.getNewInstance(po)
				.serachInLdap(organizationalUnitArray, searchFilter,
						maxRecords, returnType);
	}

	/**
	 * Wyszukiwanie w bazie dowolnego repozytorium LDAP na podstawie kryteriów
	 * zadanych w postaci filtra LDAP.
	 * 
	 * @see org.scbpm.lib.core.search.UserSearcher
	 * 
	 * @param ldapOptions
	 *            XML konfiguracyjny w postaci łańcucha znakowego.
	 * @param providerURL
	 *            adres serwera LDAP w formacie URL np.
	 *            {@code ldap://dc.ad.pl:389/}
	 * @param userCtxDN
	 *            podstawowe drzewo wyszukiwania entry użytkowników
	 * @param bindDN
	 *            DN użytkownika nawiązującego połączenie
	 * @param bindCredential
	 *            hasło użytkownika nawiązującego połączenie
	 * @param userNameAttribute
	 *            nazwa atrybutu LDAP przechowującego nazwę użytkownika.
	 * @param userDisplayNameAttribute
	 *            nazwa atrybutu LDAP przechowującego nazwę prezentacyjną
	 *            (pełną) użytkownika.
	 * @param userDNAttribute
	 *            Nazwa atrybutu definiującego 'distinguishedName' użytkownika
	 *            (lokalizację entry w drzewie katalogowym LDAP'a) - dla AD jest
	 *            to 'distinguishedName'.
	 * @param userAvatarAttribute
	 *            nazwa atrybutu przechowującego avatar użytkownika.
	 * @param searchFilter
	 *            kryterium wyszukiwania w postaci filtra LDAP
	 * @param maxRecords
	 *            maksymalna liczba rekordów zwrócona w wyniku.
	 * @param pageSize
	 *            rozmiar strony przesyłanych wyników z LDAP.
	 * @param returnType
	 *            typ do jakiego ma być skonwertowany wynik
	 * @return wynik odpowiedniego typu
	 * @throws Exception
	 */
	public static Object serachInLdap(String ldapOptions, String providerURL,
			String userCtxDN, String bindDN, String bindCredential,
			String userNameAttribute, String userDisplayNameAttribute,
			String userDNAttribute, String userAvatarAttribute,
			String searchFilter, Integer maxRecords, Integer pageSize,
			String returnType) throws Exception, FactoryNewInstanceException {
		ProviderOptions po = ProviderOptions.getNewInstance(ldapOptions);
		return UserSearcherFactory
				.getFactory()
				.getNewInstance(po)
				.serachInLdap(providerURL, userCtxDN, bindDN, bindCredential,
						userNameAttribute, userDisplayNameAttribute,
						userDNAttribute, userAvatarAttribute, searchFilter,
						maxRecords, pageSize, returnType);
	}
}
